# spdx-license-identifier: gpl-3.0-only
# Copyright (C) 2022 Michał Góral

import sys
import argparse
import logging

from ns.config import read_config, merge_args
from ns.send import send


log = logging.getLogger(__name__)


def prepare_args():
    parser = argparse.ArgumentParser(
        description="Send notifications through ntfy.sh service"
    )

    parser.add_argument("topic", help="subscribed topic")
    parser.add_argument(
        "message",
        nargs="?",
        help="message text or path to file which should be attached",
    )

    msg_group = parser.add_argument_group("message options")
    msg_group.add_argument("-T", "--title", help="message title")
    msg_group.add_argument(
        "-p", "--priority", type=int, choices=(1, 2, 3, 4, 5), help="message priority"
    )
    msg_group.add_argument(
        "-t",
        "--tag",
        dest="tags",
        action="append",
        default=[],
        help="message tags (can be specified many times",
    )
    msg_group.add_argument("-a", "--attach", metavar="URL", help="attach URL")
    msg_group.add_argument("-i", "--icon", metavar="URL", help="message icon")
    msg_group.add_argument(
        "-c", "--click", metavar="URL", help="URL which should open when clicked"
    )
    msg_group.add_argument(
        "-A",
        "--action",
        dest="actions",
        action="append",
        default=[],
        metavar="DEFINITION",
        help="message action buttons. DEFINITION should follows ntfy.sh HTTP header format for actions; can be used many times",
    )
    msg_group.add_argument("--at", help="delay notification by/until a given time")

    srv_group = parser.add_argument_group("server options")
    srv_group.add_argument(
        "-s",
        "--server",
        metavar="URL",
        help="change default ntfy server address",
    )
    srv_group.add_argument(
        "-U", "--username", help="username used to authenticate to the server"
    )
    srv_group.add_argument(
        "-P", "--password", help="password used to authenticate to the server"
    )

    return parser.parse_args()


def main():
    args = prepare_args()
    logging.basicConfig(format="%(message)s", level="INFO")

    config = read_config()
    merge_args(config, args)
    config.exec_commands()

    return not send(args, config)

    # requests.post()


sys.exit(main())



# spdx-license-identifier: gpl-3.0-only
# Copyright (C) 2022 Michał Góral

from typing import Any, Optional, Dict, Union

import os
import base64
import logging
from urllib.parse import urljoin
from argparse import Namespace

import requests

from ns.config import Config


log = logging.getLogger(__name__)


def get_auth(username: str, password: str) -> bytes:
    if username or password:
        creds = f"{username}:{password}"
        return base64.b64encode(creds.encode("utf-8"))
    return None


def set_header(name: str, headers: Dict[str, Union[str, bytes]], val: Optional[Any]):
    if val is not None:
        headers[name] = val


def send(args: Namespace, c: Config) -> bool:
    url = urljoin(c.server, args.topic)

    headers = {}
    auth = get_auth(c.username, c.password)
    if auth:
        headers["Authorization"] = "Basic ".encode("utf-8") + auth

    if args.priority:
        headers["Priority"] = str(args.priority)

    if args.title:
        headers["Title"] = str(args.title)

    if args.tags:
        headers["Tags"] = ",".join(args.tags)

    if args.attach:
        headers["Attach"] = args.attach

    if args.icon:
        headers["Icon"] = args.icon

    if args.click:
        headers["Click"] = args.click

    if args.at:
        headers["At"] = args.at

    if args.actions:
        headers["Actions"] = ";".join(args.actions)

    # message is a local path and should be treated as attachment
    if args.message and os.path.exists(args.message):
        with open(args.message, "rb") as mf:
            headers["Filename"] = os.path.basename(args.message)
            args.message = mf.read()

    kw = {"headers": headers}
    if args.message:
        kw["data"] = args.message
    resp = requests.post(url, **kw)

    if resp.text:
        log.info(resp.text.strip())

    if not resp.ok:
        log.error("Sending notification failed")
        return False

    return True



# spdx-license-identifier: gpl-3.0-only
# Copyright (C) 2022 Michał Góral

from typing import Optional

import os
import sys
import subprocess
import logging
from dataclasses import dataclass
from argparse import Namespace

import tomlkit


log = logging.getLogger(__name__)


def _run_cmd(cmd: str) -> Optional[str]:
    try:
        cp = subprocess.run(cmd, text=True, capture_output=True, check=True, shell=True)
    except subprocess.CalledProcessError:
        log.error("Command failed: %s", cmd)
        return None
    return cp.stdout


@dataclass
class Config:
    server: str = "https://ntfy.sh"
    username: Optional[str] = None
    password: Optional[str] = None

    def exec_commands(self):
        for name in self.__dataclass_fields__:
            val = getattr(self, name, None)
            if isinstance(val, str):
                val = val.strip()
                if len(val) > 2 and val[0] == "`" and val[-1] == "`":
                    new_val = _run_cmd(val[1:-1])
                    if new_val is None:
                        sys.exit(1)
                    setattr(self, name, new_val)


def merge_args(config: Config, args: Namespace):
    for name in config.__dataclass_fields__:
        attr_val = getattr(args, name, None)
        if attr_val is not None:
            setattr(config, name, attr_val)


def read_config() -> Config:
    home = os.path.expanduser("~")
    config_home = os.getenv("XDG_CONFIG_HOME", os.path.join(home, ".config"))
    path = os.path.join(config_home, "ntfy-send", "config.toml")

    try:
        with open(path, encoding="utf-8") as cfgf:
            doc = tomlkit.parse(cfgf.read())
    except FileNotFoundError:
        doc = {}

    return Config(**doc)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['ns']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.28.1,<3.0.0', 'tomlkit>=0.11.6,<0.12.0']

entry_points = \
{'console_scripts': ['ntfy-send = ns.app:main']}

setup_kwargs = {
    'name': 'ntfy-send',
    'version': '0.1.2',
    'description': 'Client application for ntfy.sh',
    'long_description': '# ntfy-send\n\nntfy-send is a client application for notification service\n[ntfy.sh](https://ntfy.sh).\n\n## Configuration\n\nntfy-send can be configured via a configuration file. It is recommended\nmethod of storing your credential informations, which simplifies obtaining\nand sending them to ntfy server.\n\nAll configuration options can be enclosed in backticks (`). When this is the\ncase, the option is treated as a command whose output substitutes the\nconfigurration option.\n\nConfiguration files are stored in _$XDG_CONFIG_HOME/ntfy-send/config.toml_.\nIf you don\'t have _$XDG_CONFIG_HOME_ environment variale set, then it is\nstored in _~/.config/ntfy-send/config.toml_. Below are documented all\noptions:\n\n```toml\n# config.toml\n\n# URL to the server\nserver = "https://ntfy.sh"\n\n# Username and password can be automatically obtained each time they\'re\n# required. This is done by passing commands which should echo credentials.\n# For complex commands it\'s recommended to pot them in a separate script, due\n# to problems with several levels of quote escaping\n\n# Username and password can be passed in plain text. This isn\'t recommended.\nusername = "user"\npassword = "pass"\n\n# Alternatively, ntfy-send can automatically run a command for username and\n# password when they\'re enclosed in backticks (`):\nusername = """`gpg2 --decrypt pass.gpg | awk -F ":" \'/user:/ { printf $2 }\'`"""\npassword = """`gpg2 --decrypt pass.gpg | awk -F ":" \'/password:/ { printf $2 }\'`"""\n```\n\n',
    'author': 'Michal Goral',
    'author_email': 'dev@goral.net.pl',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://git.goral.net.pl/mgoral/ntfy-send',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
